@@ -18,7 +18,6 @@
 #include "drawer.h"
 #include "launcher.h"
 #include "logout.h"
-#include "menu-properties.h"
 #include "menu-util.h"
 #include "menu.h"
 #include "panel-config.h"
@@ -70,8 +69,9 @@ move_applet_callback (GtkWidget *widget, AppletInfo *info)
 					PW_DRAG_OFF_CENTER);
 }
 
-static void
-panel_applet_clean_gconf (AppletInfo *info,
+void
+panel_applet_clean_gconf (AppletType  type,
+			  const char *gconf_key,
 			  gboolean    clean_gconf)
 {
 	GConfClient *client;
@@ -79,13 +79,12 @@ panel_applet_clean_gconf (AppletInfo *info,
 	const char  *temp_key = NULL;
 	const char  *profile;
 
-	g_return_if_fail (info != NULL);
-	g_return_if_fail (info->gconf_key != NULL);
+	g_return_if_fail (gconf_key != NULL);
 
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
-	if (info->type == APPLET_BONOBO)
+	if (type == APPLET_BONOBO)
 		temp_key = panel_gconf_general_key (profile, "applet_id_list");
 	else
 		temp_key = panel_gconf_general_key (profile, "object_id_list");
@@ -95,7 +94,7 @@ panel_applet_clean_gconf (AppletInfo *info,
 			GCONF_VALUE_STRING, NULL);
 
         for (l = id_list; l; l = l->next)
-                if (strcmp (info->gconf_key, (char *) l->data) == 0)
+                if (strcmp (gconf_key, (char *) l->data) == 0)
                         break;
 
         if (l) {
@@ -105,23 +104,20 @@ panel_applet_clean_gconf (AppletInfo *info,
 
                 gconf_client_set_list (client, temp_key, GCONF_VALUE_STRING, id_list, NULL);
 
-		if (info->type == APPLET_BONOBO)
+		if (type == APPLET_BONOBO)
 			temp_key = panel_gconf_sprintf (
 					"/apps/panel/profiles/%s/applets/%s",
-					profile, info->gconf_key);
+					profile, gconf_key);
 		else
 			temp_key = panel_gconf_sprintf (
 					"/apps/panel/profiles/%s/objects/%s",
-					profile, info->gconf_key);
+					profile, gconf_key);
 
 		if (clean_gconf)
 			panel_gconf_clean_dir (client, temp_key);
         }
 
 	panel_g_slist_deep_free (id_list);
-
-	g_free (info->gconf_key);
-	info->gconf_key = NULL;
 }
 
 /*destroy widgets and call the above cleanup function*/
@@ -147,7 +143,10 @@ panel_applet_clean (AppletInfo *info,
 
 	info->data = NULL;
 
-	panel_applet_clean_gconf (info, clean_gconf);
+	panel_applet_clean_gconf (info->type, info->gconf_key, clean_gconf);
+
+	g_free (info->gconf_key);
+	info->gconf_key = NULL;
 
 	queued_position_saves =
 		g_slist_remove (queued_position_saves, info);
@@ -180,33 +179,21 @@ applet_remove_callback (GtkWidget  *widget,
 }
 
 static void
-applet_callback_callback(GtkWidget *widget, gpointer data)
+applet_callback_callback (GtkWidget      *widget,
+			  AppletUserMenu *menu)
 {
-	AppletUserMenu *menu = data;
+	g_return_if_fail (menu->info != NULL);
 
-	g_return_if_fail(menu->info != NULL);
-
-	switch(menu->info->type) {
+	switch (menu->info->type) {
 	case APPLET_LAUNCHER:
-		if (strcmp (menu->name, "properties") == 0) {
+		if (!strcmp (menu->name, "properties"))
 			launcher_properties (menu->info->data);
-		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help ("launchers", NULL);
-		} else if (strcmp (menu->name, "help_on_app") == 0) {
-			Launcher * launcher = menu->info->data;
-			if (launcher->ditem != NULL) {
-				GError *error = NULL;
-				const char *docpath =
-					gnome_desktop_item_get_string
-					    (launcher->ditem, "DocPath");
-				if ( ! panel_show_gnome_kde_help (docpath, &error)) {
-					panel_error_dialog ("cannot_show_gnome_kde_help",
-							    _("<b>Cannot display help document</b>\n\n"
-							      "Details: %s"), error->message);
-					g_clear_error (&error);
-				}
-			}
-		}
+
+		else if (!strcmp (menu->name, "help"))
+			panel_show_help ("wgospanel.xml", "gospanel-16");
+
+		else if (!strcmp (menu->name, "help_on_app"))
+			launcher_show_help (menu->info->data);
 		break;
 	case APPLET_DRAWER: 
 		if (strcmp (menu->name, "properties")==0) {
@@ -214,24 +201,12 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 			g_assert(drawer);
 			panel_config(drawer->drawer);
 		} else if (strcmp (menu->name, "help") == 0) {
-			panel_show_help ("drawers", NULL);
+			panel_show_help ("wgospanel.xml", "gospanel-18");
 		}
 		break;
 	case APPLET_MENU:
-		if(strcmp(menu->name,"properties")==0)
-			menu_properties(menu->info->data);
-		else if(strcmp(menu->name,"edit_menus")==0) {
-			char *tmp[3] = { NULL, NULL, NULL };
-			if((tmp[0] = g_find_program_in_path ("nautilus")) != NULL)  {
-				tmp[1] = "applications:/";
-				gnome_execute_async (g_get_home_dir (), 2, tmp);
-				g_free (tmp[0]);
-			}
-		} else if (strcmp (menu->name, "help") == 0) {
-			Menu *menu2 = menu->info->data;
-			char *page;
-			page = menu2->main_menu ? "mainmenu" : "menus";
-			panel_show_help (page, NULL);
+		if (strcmp (menu->name, "help") == 0) {
+			panel_show_help ("wgospanel.xml", "gospanel-37");
 		}
 		break;
 	case APPLET_LOCK: {
@@ -249,7 +224,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 		char *command = NULL;
 		gboolean freeit = FALSE;
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "LOCKBUTTON");
+			panel_show_help ("wgospanel.xml", "gospanel-21");
 		else if (strcmp (menu->name, "restart") == 0) {
 			command = "xscreensaver-command -exit ; xscreensaver &";
 		} else if (strcmp (menu->name, "prefs") == 0) {
@@ -267,7 +242,7 @@ applet_callback_callback(GtkWidget *widget, gpointer data)
 	}
 	case APPLET_LOGOUT:
 		if (strcmp (menu->name, "help") == 0)
-			panel_show_help ("specialobjects", "LOGOUTBUTTON");
+			panel_show_help ("wgospanel.xml", "gospanel-20");
 		break;
 	case APPLET_BONOBO:
 		/*
@@ -409,16 +384,16 @@ setup_an_item(AppletUserMenu *menu,
 		gtk_menu_shell_append (GTK_MENU_SHELL (submenu), menu->menuitem);
 
 	/*if an item not a submenu*/
-	if(!is_submenu) {
-		g_signal_connect(G_OBJECT(menu->menuitem), "activate",
-				 G_CALLBACK (applet_callback_callback),
-				   menu);
-		g_signal_connect(G_OBJECT (submenu), "destroy",
-				 G_CALLBACK (gtk_widget_destroyed),
-				 &menu->submenu);
+	if (!is_submenu) {
+		g_signal_connect (menu->menuitem, "activate",
+				  G_CALLBACK (applet_callback_callback),
+				  menu);
+		g_signal_connect (submenu, "destroy",
+				  G_CALLBACK (gtk_widget_destroyed),
+				  &menu->submenu);
 	/* if the item is a submenu and doesn't have it's menu
 	   created yet*/
-	} else if(!menu->submenu) {
+	} else if (!menu->submenu) {
 		menu->submenu = panel_menu_new ();
 	}
 
@@ -535,7 +510,7 @@ panel_applet_create_menu (AppletInfo *info)
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
 						  GTK_ICON_SIZE_MENU);
 
-		setup_menuitem (menuitem, image , _("Remove From Panel"));
+		setup_menuitem (menuitem, image , _("_Remove From Panel"));
 
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
@@ -546,7 +521,7 @@ panel_applet_create_menu (AppletInfo *info)
 
 		/* FIXME: should have a "Move" pixmap.
 		 */
-		setup_menuitem (menuitem, NULL, _("Move"));
+		setup_menuitem (menuitem, NULL, _("_Move"));
 
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
@@ -683,7 +658,9 @@ applet_destroy (GtkWidget *w, AppletInfo *info)
 
 		location = gnome_desktop_item_get_location (launcher->ditem);
 
-		session_add_dead_launcher (location);
+		/* Launcher may not yet have been hoarded */
+		if (location)
+			session_add_dead_launcher (location);
 	}
 
 
@@ -815,7 +792,11 @@ panel_applet_load_from_unique_id (AppletType   type,
 	temp_key = panel_applet_get_full_gconf_key (
 			type, profile, unique_id, "object_type");
 	type_string = gconf_client_get_string (gconf_client, temp_key, NULL);
-
+	if (!type_string) {
+		g_printerr (_("No object_type set for panel object with ID %s\n"), unique_id);
+		return;
+	}
+        
 	if (!gconf_string_to_enum (object_type_enum_map,
 				   type_string,
 				   (int *) &applet_type)) {
@@ -833,7 +814,11 @@ panel_applet_load_from_unique_id (AppletType   type,
 	temp_key = panel_applet_get_full_gconf_key (
 			type, profile, unique_id, "panel_id");
 	panel_id = gconf_client_get_string (gconf_client, temp_key, NULL);
-
+	if (!panel_id) {
+		g_printerr (_("No panel_id set for panel object with ID %s\n"), unique_id);
+		return;
+	}
+        
 	temp_key = panel_applet_get_full_gconf_key (
 			type, profile, unique_id, "panel_right_stick");
 	right_stick = gconf_client_get_bool (gconf_client, temp_key, NULL);
@@ -841,10 +826,8 @@ panel_applet_load_from_unique_id (AppletType   type,
 	panel_widget = panel_widget_get_by_id (panel_id);
 	g_free (panel_id);
 
-	if (!panel_widget) {
-		g_warning ("panel_applet_load: can't find the panel for this object\n");
+	if (!panel_widget)
 		return;
-	}
 
 	if (right_stick)
 		position = panel_widget->size - position;
@@ -971,7 +954,7 @@ panel_applet_save_position (AppletInfo *applet_info,
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
 
-	right_stick = panel_applet_get_right_stick (applet_info);
+	right_stick = panel_is_applet_right_stick (applet_info->widget);
 	temp_key = panel_applet_get_full_gconf_key (
 			applet_info->type, profile, gconf_key, "panel_right_stick");
 	gconf_client_set_bool (client, temp_key, right_stick, NULL);
@@ -1187,6 +1170,11 @@ panel_applet_register (GtkWidget      *applet,
 
 	panel_applet_save_to_gconf (info);
 
+	if (type != APPLET_BONOBO)
+		gtk_widget_grab_focus (applet);
+	else
+		gtk_widget_child_focus (applet, GTK_DIR_TAB_FORWARD);
+
 	return info;
 }
 
@@ -1215,16 +1203,3 @@ panel_applet_get_panel_id (AppletInfo *applet)
 
 	return panel->unique_id;
 }
-
-gboolean
-panel_applet_get_right_stick (AppletInfo *applet)
-{
-	PanelWidget *panel;
-
-	g_return_val_if_fail (applet != NULL, FALSE);
-	g_return_val_if_fail (G_IS_OBJECT (applet->widget), FALSE);
-
-	panel = PANEL_WIDGET (applet->widget->parent);
-
-	return panel_widget_is_applet_stuck (panel, applet->widget);
-}
